@@ -29,19 +29,35 @@ using namespace nonstd;
 
 struct Implicit { int x;          Implicit(int v) : x(v) {} };
 struct Explicit { int x; explicit Explicit(int v) : x(v) {} };
-struct MoveOnly {
+
+struct MoveOnly
+{
     int x;
     explicit MoveOnly(int x) :x{x} {}
-    MoveOnly(const MoveOnly&) = delete;
-    MoveOnly(MoveOnly&& other) noexcept :x{other.x} {}
-    MoveOnly& operator=(const MoveOnly&) = delete;
-    MoveOnly& operator=(MoveOnly&& other) noexcept {
-        if (&other == this) return *this;
+
+    MoveOnly( MoveOnly const &  ) = delete;
+    MoveOnly( MoveOnly && other ) noexcept : x{ other.x } {}
+
+    MoveOnly& operator=( MoveOnly const &  ) = delete;
+    MoveOnly& operator=( MoveOnly && other ) noexcept
+    {
+        if (&other == this)
+            return *this;
         x = other.x;
         return *this;
     }
 };
 
+struct NonMovableNonCopyable
+{
+    NonMovableNonCopyable() = default;
+
+    NonMovableNonCopyable( NonMovableNonCopyable const & ) = delete;
+    NonMovableNonCopyable( NonMovableNonCopyable &&      ) = delete;
+    NonMovableNonCopyable& operator=( NonMovableNonCopyable const & ) = delete;
+    NonMovableNonCopyable& operator=( NonMovableNonCopyable &&      ) = delete;
+};
+
 bool operator==( Implicit a, Implicit b ) { return a.x == b.x; }
 bool operator==( Explicit a, Explicit b ) { return a.x == b.x; }
 
@@ -175,8 +191,8 @@ CASE( "unexpected_type: Allows to copy-construct from unexpected_type, default"
 
     unexpected_type<int> b( a );
 
-    EXPECT( a.value() == 7 );
-    EXPECT( b.value() == 7 );
+    EXPECT( a.error() == 7 );
+    EXPECT( b.error() == 7 );
 }
 
 CASE( "unexpected_type: Allows to move-construct from unexpected_type, default" )
@@ -185,8 +201,8 @@ CASE( "unexpected_type: Allows to move-construct from unexpected_type, default"
 
     unexpected_type<int> b( std::move( a ) );
 
-    EXPECT( a.value() == 7 );
-    EXPECT( b.value() == 7 );
+    EXPECT( a.error() == 7 );
+    EXPECT( b.error() == 7 );
 }
 
 CASE( "unexpected_type: Allows to in-place-construct" )
@@ -194,78 +210,94 @@ CASE( "unexpected_type: Allows to in-place-construct" )
     unexpected_type<Explicit> ue( in_place, 5 );
     unexpected_type<Implicit> ui( in_place, 7 );
 
-    EXPECT( ue.value() == Explicit{5} );
-    EXPECT( ui.value() == Implicit{7} );
+    EXPECT( ue.error() == Explicit{5} );
+    EXPECT( ui.error() == Implicit{7} );
 }
 
 CASE( "unexpected_type: Allows to in-place-construct from initializer_list" )
 {
     unexpected_type<InitList> u( in_place, { 7, 8, 9 }, 'a' );
 
-    EXPECT( u.value().vec[0]  ==  7 );
-    EXPECT( u.value().vec[1]  ==  8 );
-    EXPECT( u.value().vec[2]  ==  9 );
-    EXPECT( u.value().c       == 'a');
+    EXPECT( u.error().vec[0]  ==  7 );
+    EXPECT( u.error().vec[1]  ==  8 );
+    EXPECT( u.error().vec[2]  ==  9 );
+    EXPECT( u.error().c       == 'a');
 }
 
 CASE( "unexpected_type: Allows to copy-construct from error_type" )
 {
     unexpected_type<int> u{ 7 };
 
-    EXPECT( u.value() == 7 );
+    EXPECT( u.error() == 7 );
 }
 
 CASE( "unexpected_type: Allows to move-construct from error_type" )
 {
     unexpected_type<int> u{ std::move( 7 ) };
 
-    EXPECT( u.value() == 7 );
+    EXPECT( u.error() == 7 );
 }
 
 CASE( "unexpected_type: Allows to copy-construct from unexpected_type, explicit converting" )
 {
+#if !nsel_USES_STD_EXPECTED // TODO
     unexpected_type<int> a{ 7 };
 
     unexpected_type<Explicit> b{ a };
 
-    EXPECT( b.value() == Explicit{7} );
+    EXPECT( b.error() == Explicit{7} );
+#else
+    EXPECT( !!"no explicit converting copy-construct for std::unexpected (C++23)." "TODO" );
+#endif
 }
 
 CASE( "unexpected_type: Allows to copy-construct from unexpected_type, non-explicit converting" )
 {
+#if !nsel_USES_STD_EXPECTED // TODO
     unexpected_type<int> a{ 7 };
 
     unexpected_type<Implicit> b( a );
 
-    EXPECT( b.value() == Implicit{7} );
+    EXPECT( b.error() == Implicit{7} );
+#else
+    EXPECT( !!"no non-explicit converting copy-construct for std::unexpected (C++23)." "TODO" );
+#endif
 }
 
 CASE( "unexpected_type: Allows to move-construct from unexpected_type, explicit converting" )
 {
+#if !nsel_USES_STD_EXPECTED // TODO
     unexpected_type<int> a{ 7 };
 
     unexpected_type<Explicit> b{ std::move( a ) };
 
-    EXPECT( b.value() == Explicit{7} );
+    EXPECT( b.error() == Explicit{7} );
+#else
+    EXPECT( !!"no explicit converting move-construct for std::unexpected (C++23)." "TODO" );
+#endif
 }
 
 CASE( "unexpected_type: Allows to move-construct from unexpected_type, non-explicit converting" )
 {
+#if !nsel_USES_STD_EXPECTED // TODO
     unexpected_type<int> a{ 7 };
 
     unexpected_type<Implicit> b( std::move( a ) );
 
-    EXPECT( b.value() == Implicit{7} );
+    EXPECT( b.error() == Implicit{7} );
+#else
+    EXPECT( !!"no non-explicit converting move-construct for std::unexpected (C++23)." "TODO" );
+#endif
 }
 
 CASE( "unexpected_type: Allows to copy-assign from unexpected_type, default" )
 {
-    unexpected_type<int> a{ 7   };
+    unexpected_type<int> a{ 7 };
     unexpected_type<int> b{ 0 };
 
     b = a;
 
-    EXPECT( b.value() == 7 );
+    EXPECT( b.error() == 7 );
 }
 
 CASE( "unexpected_type: Allows to move-assign from unexpected_type, default" )
@@ -275,11 +307,12 @@ CASE( "unexpected_type: Allows to move-assign from unexpected_type, default" )
 
     b = std::move( a );
 
-    EXPECT( b.value() == 7 );
+    EXPECT( b.error() == 7 );
 }
 
 CASE( "unexpected_type: Allows to copy-assign from unexpected_type, converting" )
 {
+#if !nsel_USES_STD_EXPECTED
     unexpected_type<int> u{ 7 };
     unexpected_type<Explicit> ue{ 0 };
     unexpected_type<Implicit> ui{ 0 };
@@ -287,12 +320,16 @@ CASE( "unexpected_type: Allows to copy-assign from unexpected_type, converting"
     ue = u;
     ui = u;
 
-    EXPECT( ue.value() == Explicit{7} );
-    EXPECT( ui.value() == Implicit{7} );
+    EXPECT( ue.error() == Explicit{7} );
+    EXPECT( ui.error() == Implicit{7} );
+#else
+    EXPECT( !!"no copy-assignment for std::unexpected (C++23)." );
+#endif
 }
 
 CASE( "unexpected_type: Allows to move-assign from unexpected, converting" )
 {
+#if !nsel_USES_STD_EXPECTED
     unexpected_type<int> u{ 7 };
     unexpected_type<int> v{ 7 };
     unexpected_type<Explicit> ue{ 0 };
@@ -301,8 +338,11 @@ CASE( "unexpected_type: Allows to move-assign from unexpected, converting" )
     ue = std::move( u );
     ui = std::move( v );
 
-    EXPECT( ue.value() == Explicit{7} );
-    EXPECT( ui.value() == Implicit{7} );
+    EXPECT( ue.error() == Explicit{7} );
+    EXPECT( ui.error() == Implicit{7} );
+#else
+    EXPECT( !!"no move-assignment for std::unexpected (C++23)." );
+#endif
 }
 
 CASE( "unexpected_type: Allows to observe its value via a l-value reference" )
@@ -310,8 +350,8 @@ CASE( "unexpected_type: Allows to observe its value via a l-value reference" )
     unexpected_type<int>  u{ 7 };
     unexpected_type<int> uc{ 7 };
 
-    EXPECT(  u.value() == 7 );
-    EXPECT( uc.value() == 7 );
+    EXPECT(  u.error() == 7 );
+    EXPECT( uc.error() == 7 );
 }
 
 CASE( "unexpected_type: Allows to observe its value via a r-value reference" )
@@ -319,17 +359,17 @@ CASE( "unexpected_type: Allows to observe its value via a r-value reference" )
     unexpected_type<int>  u{ 7 };
     unexpected_type<int> uc{ 7 };
 
-    EXPECT( std::move( u).value() == 7 );
-    EXPECT( std::move(uc).value() == 7 );
+    EXPECT( std::move( u).error() == 7 );
+    EXPECT( std::move(uc).error() == 7 );
 }
 
 CASE( "unexpected_type: Allows to modify its value via a l-value reference" )
 {
     unexpected_type<int> u{ 5 };
 
-    u.value() = 7;
+    u.error() = 7;
 
-    EXPECT( u.value() == 7 );
+    EXPECT( u.error() == 7 );
 }
 
 //CASE( "unexpected_type: Allows to modify its value via a r-value reference" )
@@ -337,9 +377,9 @@ CASE( "unexpected_type: Allows to modify its value via a l-value reference" )
 //    const auto v = 9;
 //    unexpected_type<int> u{ 7 };
 //
-//    std::move( u.value() ) = v;
+//    std::move( u.error() ) = v;
 //
-//    EXPECT( u.value() == v );
+//    EXPECT( u.error() == v );
 //}
 
 CASE( "unexpected_type: Allows to be swapped" )
@@ -349,8 +389,8 @@ CASE( "unexpected_type: Allows to be swapped" )
 
     a.swap( b );
 
-    EXPECT( a.value() == 7 );
-    EXPECT( b.value() == 5 );
+    EXPECT( a.error() == 7 );
+    EXPECT( b.error() == 5 );
 }
 
 //CASE( "unexpected_type: Allows reset via = {}" )
@@ -396,7 +436,7 @@ CASE( "unexpected_type<std::exception_ptr>: Allows to copy-construct from error_
 
     unexpected_type<std::exception_ptr> u{ ep };
 
-    EXPECT( u.value() == ep );
+    EXPECT( u.error() == ep );
 }
 
 CASE( "unexpected_type<std::exception_ptr>: Allows to move-construct from error_type" )
@@ -406,7 +446,7 @@ CASE( "unexpected_type<std::exception_ptr>: Allows to move-construct from error_
 
     unexpected_type<std::exception_ptr> u{ std::move( ep_move ) };
 
-    EXPECT( u.value() == ep_copy );
+    EXPECT( u.error() == ep_copy );
 }
 
 CASE( "unexpected_type<std::exception_ptr>: Allows to copy-construct from an exception" )
@@ -417,7 +457,7 @@ CASE( "unexpected_type<std::exception_ptr>: Allows to copy-construct from an exc
 
     try
     {
-         std::rethrow_exception( u.value() );
+         std::rethrow_exception( u.error() );
     }
     catch( std::exception const & e )
     {
@@ -430,7 +470,7 @@ CASE( "unexpected_type<std::exception_ptr>: Allows to observe its value" )
     const auto ep = make_ep();
     unexpected_type<std::exception_ptr> u{ ep };
 
-    EXPECT( u.value() == ep );
+    EXPECT( u.error() == ep );
 }
 
 CASE( "unexpected_type<std::exception_ptr>: Allows to modify its value" )
@@ -439,9 +479,9 @@ CASE( "unexpected_type<std::exception_ptr>: Allows to modify its value" )
     const auto ep2 = make_ep();
     unexpected_type<std::exception_ptr> u{ ep1 };
 
-    u.value() = ep2;
+    u.error() = ep2;
 
-    EXPECT( u.value() == ep2 );
+    EXPECT( u.error() == ep2 );
 }
 
 //CASE( "unexpected_type: Allows reset via = {}, std::exception_ptr specialization" )
@@ -523,7 +563,7 @@ CASE( "make_unexpected(): Allows to create an unexpected_type<E> from an E" )
     const auto error = 7;
     auto u = make_unexpected( error );
 
-    EXPECT( u.value() == error );
+    EXPECT( u.error() == error );
 }
 
 CASE( "make_unexpected_from_current_exception(): Allows to create an unexpected_type<std::exception_ptr> from the current exception" "[.deprecated]" )
@@ -541,7 +581,7 @@ CASE( "make_unexpected_from_current_exception(): Allows to create an unexpected_
 
         try
         {
-            std::rethrow_exception( u.value() );
+            std::rethrow_exception( u.error() );
         }
         catch( std::exception const & e )
         {
@@ -619,6 +659,20 @@ CASE( "expected: Allows to default construct" )
     EXPECT( e.has_value() );
 }
 
+CASE( "expected: Allows to default construct from noncopyable, noncopyable value type" )
+{
+    expected<NonMovableNonCopyable, NonMovableNonCopyable> e;
+
+    EXPECT( e.has_value() );
+}
+
+CASE( "expected: Allows to default construct from noncopyable, noncopyable error type" )
+{
+    expected<NonMovableNonCopyable, NonMovableNonCopyable> e{ unexpect_t{} };
+
+    EXPECT( !e.has_value() );
+}
+
 CASE( "expected: Allows to copy-construct from expected: value" )
 {
     expected<int, char> a = 7;
@@ -945,7 +999,7 @@ CASE( "expected: Allows to emplace value" )
 
 CASE( "expected: Allows to emplace value from initializer_list" )
 {
-    expected<InitList, char> e{ {}, 'x'};
+    expected<InitList, char> e{ in_place, std::initializer_list<int>{}, 'x'};
 
     auto ve = e.emplace( { 7, 8, 9 }, 'a' );
 
@@ -1158,18 +1212,26 @@ CASE( "expected: Allows to move its error" )
 
 CASE( "expected: Allows to observe its error as unexpected" )
 {
+#if !nsel_USES_STD_EXPECTED
     const auto v = 7;
     expected<char, Implicit> e{ unexpect, v };
 
-    EXPECT( e.get_unexpected().value() == v );
+    EXPECT( e.get_unexpected().error() == v );
+#else
+    EXPECT( !!"expected::get_unexpected() is not available (using C++23 std::expected)" );
+#endif
 }
 
 CASE( "expected: Allows to query if it contains an exception of a specific base type" )
 {
+#if !nsel_USES_STD_EXPECTED
     expected<int, std::out_of_range> e{ unexpect, std::out_of_range( "oor" ) };
 
     EXPECT(  e.has_exception< std::logic_error   >() );
     EXPECT( !e.has_exception< std::runtime_error >() );
+#else
+    EXPECT( !!"expected::has_exception() is not available (using C++23 std::expected)" );
+#endif
 }
 
 CASE( "expected: Allows to observe its value if available, or obtain a specified value otherwise" )
@@ -1210,6 +1272,128 @@ CASE( "expected: Throws bad_expected_access on value access when disengaged" )
     EXPECT_THROWS_AS( std::move(ec).value(), bad_expected_access<int> );
 }
 
+#if nsel_P2505R >= 4
+CASE( "expected: Allows to observe its unexpected value, or fallback to the specified value with error_or" " [monadic p2505r4]")
+{
+    const auto ve = 3;
+    const auto vu = 7;
+    expected<int, int> e{ ve };
+    expected<int, int> u{ unexpect, 0 };
+
+    EXPECT( e.error_or( vu ) == vu );
+    EXPECT( u.error_or( vu ) ==  0 );
+}
+#endif // nsel_P2505R >= 4
+
+#if nsel_P2505R >= 3
+CASE( "expected: Allows to map value with and_then" " [monadic p2505r3]" )
+{
+    const auto mul2 = []( int n ) -> expected<int, int> { return n * 2; };
+    const auto to_unexpect42 = []( int ) -> expected<int, int> { return make_unexpected( 42 ); };
+
+    {
+        expected<int, int> e{ 11 };
+        const expected<int, int> ce{ 21 };
+        expected<int, int> ue{ unexpect, 42 };
+        EXPECT( e.and_then( mul2 ).value() == 22 );
+        EXPECT( ce.and_then( mul2 ).value() == 42 );
+        EXPECT( !ue.and_then( mul2 ).has_value());
+        EXPECT( ue.and_then( mul2 ).error() == 42 );
+        EXPECT( !e.and_then( to_unexpect42 ).has_value());
+        EXPECT( e.and_then( to_unexpect42 ).error() == 42 );
+        EXPECT( ce.and_then( to_unexpect42 ).error() == 42 );
+    }
+
+    const auto moveonly_x_mul2 = [](MoveOnly val) -> expected<int, int> { return val.x * 2; };
+    EXPECT( (expected<MoveOnly, int>{ MoveOnly{ 33 } }).and_then( moveonly_x_mul2 ).value() == 66 );
+    EXPECT( (expected<MoveOnly, int>{ MoveOnly{ 15 } }).and_then( [](MoveOnly&&) -> expected<MoveOnly, int> { return make_unexpected( 42 ); } ).error() == 42 );
+
+    const auto map_to_void = [](int) -> expected<void, int> { return {}; };
+    const auto map_to_void_unexpect42 = [](int) -> expected<void, int> { return make_unexpected( 42 ); };
+    static_assert( std::is_same< expected<void, int>, decltype( expected<int, int>( 3 ).and_then( map_to_void ) ) >::value,
+        "and_then mapping to void results in expected<void>");
+    EXPECT( (expected<int, int>(3)).and_then( map_to_void ).has_value() );
+    EXPECT( !(expected<int, int>(3)).and_then( map_to_void_unexpect42 ).has_value() );
+    EXPECT( (expected<int, int>(3)).and_then( map_to_void_unexpect42 ).error() == 42 );
+}
+
+CASE( "expected: Allows to map unexpected with or_else" " [monadic p2505r3]" )
+{
+    const auto to_unexpect43 = []( int ) -> expected<int, int> { return make_unexpected( 43 ); };
+
+    {
+        expected<int, int> e{ 11 };
+        const expected<int, int> ce{ 21 };
+        expected<int, int> ue{ unexpect, 42 };
+        EXPECT( e.or_else( to_unexpect43 ).has_value());
+        EXPECT( e.or_else( to_unexpect43 ).value() == 11 );
+        EXPECT( ce.or_else( to_unexpect43 ).value() == 21 );
+        EXPECT( !ue.or_else( to_unexpect43 ).has_value());
+        EXPECT( ue.or_else( to_unexpect43 ).error() == 43 );
+    }
+
+    const auto fallback_throw = []( int ) -> expected<int, int> { throw std::runtime_error( "or_else" ); };
+    EXPECT_THROWS_AS( (expected<int, int>{ unexpect, 42 }).or_else( fallback_throw ), std::runtime_error );
+
+    const auto moveonly_fallback_to_66 = [](int) -> expected<MoveOnly, int> { return MoveOnly{ 66 }; };
+    EXPECT( (expected<MoveOnly, int>{ MoveOnly{ 33 } }).or_else( moveonly_fallback_to_66 ).value() == 33 );
+    EXPECT( (expected<MoveOnly, int>{ unexpect, 15 }).or_else(moveonly_fallback_to_66).value() == 66 );
+}
+
+CASE( "expected: Allows to transform value" " [monadic p2505r3]" )
+{
+    const auto mul2 = []( int n ) -> int { return n * 2; };
+
+    {
+        expected<int, int> e{ 11 };
+        const expected<int, int> ce{ 21 };
+        expected<int, int> ue{ unexpect, 42 };
+        EXPECT( e.transform( mul2 ).value() == 22 );
+        EXPECT( ce.transform( mul2 ).value() == 42 );
+        EXPECT( !ue.transform( mul2 ).has_value());
+        EXPECT( ue.transform( mul2 ).error() == 42 );
+    }
+
+#if nsel_P2505R >= 5
+    // R5 changed remove_cvref_t to remove_cv_t in transform/transform_error, which broke data member pointer transforms,
+    // because the result type must be a valid expected value type, but expressions with reference types are not.
+    const auto moveonly_map_to_x = [](MoveOnly val) { return val.x; };
+#else
+    const auto moveonly_map_to_x = &MoveOnly::x;
+#endif // nsel_P2505R >= 5
+    const auto moveonly_x_mul2 = [](MoveOnly val) -> int { return val.x * 2; };
+    EXPECT( (expected<MoveOnly, int>{ MoveOnly{ 33 } }).transform( moveonly_map_to_x ).value() == 33 );
+    EXPECT( (expected<MoveOnly, int>{ MoveOnly{ 33 } }).transform( moveonly_map_to_x ).transform( mul2 ).value() == 66 );
+    EXPECT( (expected<MoveOnly, int>{ MoveOnly{ 33 } }).transform( moveonly_x_mul2 ).has_value() );
+    EXPECT( (expected<MoveOnly, int>{ MoveOnly{ 33 } }).transform( moveonly_x_mul2 ).value() == 66 );
+    EXPECT( !(expected<MoveOnly, int>{ unexpect, 15 }).transform( [](MoveOnly&&) -> int { return 42; } ).has_value() );
+    EXPECT( (expected<MoveOnly, int>{ unexpect, 15 }).transform( [](MoveOnly&&) -> int { return 42; } ).error() == 15 );
+
+    const auto map_to_void = [](int) -> void { };
+    static_assert( std::is_same< expected<void, int>, decltype( expected<int, int>( 3 ).transform( map_to_void ) ) >::value,
+        "transform to void results in expected<void>" );
+    EXPECT( (expected<int, int>(3)).transform( map_to_void ).has_value() );
+    static_assert( std::is_same< decltype( (expected<int, int>(3)).transform( map_to_void ).value() ), void >::value,
+        "transform to void results in void value" );
+}
+
+CASE( "expected: Allows to map errors with transform_error" " [monadic p2505r3]" )
+{
+    const auto to_43 = []( int ) -> int { return 43; };
+
+    {
+        expected<int, int> e{ 11 };
+        const expected<int, int> ce{ 21 };
+        expected<int, int> ue{ unexpect, 42 };
+        EXPECT( e.transform_error( to_43 ).has_value());
+        EXPECT( e.transform_error( to_43 ).value() == 11 );
+        EXPECT( ce.transform_error( to_43 ).value() == 21 );
+        EXPECT( !ue.transform_error( to_43 ).has_value());
+        EXPECT( ue.transform_error( to_43 ).error() == 43 );
+    }
+}
+#endif // nsel_P2505R >= 3
+
 // -----------------------------------------------------------------------
 // expected<void> specialization
 
@@ -1477,18 +1661,26 @@ CASE( "expected<void>: Allows to move its error" )
 
 CASE( "expected<void>: Allows to observe its error as unexpected" )
 {
+#if !nsel_USES_STD_EXPECTED
     const auto value = 7;
     expected<void, int> e{ unexpect, value };
 
-    EXPECT( e.get_unexpected().value() == value );
+    EXPECT( e.get_unexpected().error() == value );
+#else
+    EXPECT( !!"expected::get_unexpected() is not available (using C++23 std::expected)" );
+#endif
 }
 
 CASE( "expected<void>: Allows to query if it contains an exception of a specific base type" )
 {
+#if !nsel_USES_STD_EXPECTED
     expected<void, std::out_of_range> e{ unexpect, std::out_of_range( "oor" ) };
 
     EXPECT(  e.has_exception< std::logic_error   >() );
     EXPECT( !e.has_exception< std::runtime_error >() );
+#else
+    EXPECT( !!"expected::has_exception() is not available (using C++23 std::expected)" );
+#endif
 }
 
 CASE( "expected<void>: Throws bad_expected_access on value access when disengaged" )
@@ -1507,6 +1699,123 @@ CASE( "expected<void>: Throws bad_expected_access on value access when disengage
     EXPECT_THROWS_AS( std::move(ec).value(), bad_expected_access<int> );
 }
 
+#if nsel_P2505R >= 4
+CASE( "expected<void>: Allows to observe unexpected value, or fallback to a default value with error_or" " [monadic p2505r4]" )
+{
+    const auto vu = 7;
+    expected<void, int> e;
+    expected<void, int> u{ unexpect, 0 };
+
+    EXPECT( e.error_or( vu ) == vu );
+    EXPECT( u.error_or( vu ) == 0 );
+}
+#endif // nsel_P2505R >= 4
+
+#if nsel_P2505R >= 3
+CASE( "expected<void>: Allows to call argless functions with and_then" " [monadic p2505r3]" )
+{
+    const auto ret22 = []() -> expected<int, int> { return 22; };
+    const auto unexpect32 = []() -> expected<int, int> { return make_unexpected( 32 ); };
+
+    {
+        expected<void, int> e;
+        const expected<void, int> ce;
+        expected<void, int> ue{ unexpect, 42 };
+        EXPECT( e.has_value() );
+        EXPECT( ce.has_value() );
+        EXPECT( e.and_then( ret22 ).value() == 22 );
+        EXPECT( ce.and_then( ret22 ).value() == 22 );
+        EXPECT( !ue.and_then( ret22 ).has_value());
+        EXPECT( ue.and_then( ret22 ).error() == 42 );
+        EXPECT( !e.and_then( unexpect32 ).has_value());
+        EXPECT( e.and_then( unexpect32 ).error() == 32 );
+        EXPECT( ce.and_then( unexpect32 ).error() == 32 );
+    }
+
+    {
+        bool called = false;
+        expected<void, int> e;
+        e.and_then( [&called]() -> expected<void, int> {
+            called = true;
+            return {};
+        } );
+        EXPECT( called );
+    }
+
+    {
+        bool called = false;
+        expected<void, int>{}.and_then( [&called]() -> expected<void, int> {
+            called = true;
+            return {};
+        } );
+        EXPECT( called );
+    }
+
+    {
+        bool called = false;
+        expected<void, int>{ unexpect, 42 }.and_then( [&called]() -> expected<void, int> {
+            called = true;
+            return {};
+        } );
+        EXPECT( !called );
+    }
+
+    const bool map_to_unexpect_success = !expected<void, int>{}.and_then( []() -> expected<void, int> { return make_unexpected( 42 ); } ).has_value();
+    EXPECT( map_to_unexpect_success );
+}
+
+CASE( "expected<void>: Allows to map to expected or unexpected with or_else" " [monadic p2505r3]" )
+{
+    const auto make_valid = [](int) -> expected<void, int> { return {}; };
+    const auto unexpect32 = [](int) -> expected<void, int> { return make_unexpected( 32 ); };
+
+    {
+        expected<void, int> e;
+        const expected<void, int> ce;
+        expected<void, int> ue{ unexpect, 42 };
+        EXPECT( e.has_value() );
+        EXPECT( ce.has_value() );
+        EXPECT( e.or_else( unexpect32 ).has_value());
+        static_assert( std::is_same< decltype( e.or_else( unexpect32 ).value() ), void >::value,
+            "or_else mapping to void results in void value" );
+        EXPECT( ce.or_else( unexpect32 ).has_value());
+        EXPECT( !ue.or_else( unexpect32 ).has_value());
+        EXPECT( ue.or_else( make_valid ).has_value() );
+        static_assert( std::is_same< decltype( ue.or_else( make_valid ).value() ), void >::value,
+            "or_else mapping to void results in void value" );
+        EXPECT( ue.or_else( unexpect32 ).error() == 32 );
+    }
+}
+
+CASE( "expected<void>: Allows to assign a new expected value using transform" " [monadic p2505r3]" )
+{
+    const auto make_int_32 = [] { return 32; };
+    const auto mul2 = [](int v) { return v * 2; };
+    expected<void, int> e;
+    static_assert( std::is_same< decltype( e.transform( make_int_32 ) )::value_type, int >::value, "" );
+    EXPECT( e.transform( make_int_32 ).value() == 32 );
+    EXPECT( e.transform( make_int_32 ).transform( mul2 ).value() == 64 );
+}
+
+CASE( "expected<void>: Allows to map unexpected error value via transform_error" " [monadic p2505r3]" )
+{
+    const auto mul2 = []( int v ) -> int { return v * 2; };
+    enum class my_error { einval };
+    const auto map_to_my_error = [](int) { return my_error::einval; };
+
+    {
+        expected<void, int> e;
+        const expected<void, int> ce;
+        expected<void, int> ue{ unexpect, 42 };
+        EXPECT( e.transform_error( mul2 ).has_value());
+        EXPECT( ce.transform_error( mul2 ).has_value());
+        EXPECT( !ue.transform_error( mul2 ).has_value());
+        EXPECT( ue.transform_error( mul2 ).error() == 84 );
+        EXPECT( ue.transform_error( map_to_my_error ).error() == my_error::einval );
+    }
+}
+#endif // nsel_P2505R >= 3
+
 // [expected<> unwrap()]
 
 // [expected<> factories]
@@ -1660,10 +1969,14 @@ CASE( "swap: Allows expected to be swapped" )
 
 CASE( "std::hash: Allows to compute hash value for expected" )
 {
+#if !nsel_USES_STD_EXPECTED
     expected<int, char> a{ 7 };
     expected<int, char> b{ 7 };
 
     EXPECT( (std::hash< expected<int, char> >{}( a )) == (std::hash< expected<int, char> >{}( b )) );
+#else
+    EXPECT( !!"std::hash<std::expected<>> is not available for std::unexpected (C++23)." );
+#endif
 }
 
 #if nsel_P0323R <= 3
@@ -1803,7 +2116,7 @@ public:
 
 nonstd::expected< MyNonMoveableObject, Error > create_copyable()
 {
-    return MyNonMoveableObject{};
+    return nonstd::expected< MyNonMoveableObject, Error >{};
 }
 
 class MyNonCopyableObject
@@ -1877,6 +2190,60 @@ int compare_not_equal_with_expected_void()
 
 } // namespace issue_51
 
+namespace issue_59 {
+
+struct NonMovableNonCopyable
+{
+    NonMovableNonCopyable() = default;
+
+    NonMovableNonCopyable( NonMovableNonCopyable const &  ) = delete;
+    NonMovableNonCopyable( NonMovableNonCopyable       && ) = delete;
+    NonMovableNonCopyable& operator=( NonMovableNonCopyable const &  ) = delete;
+    NonMovableNonCopyable& operator=( NonMovableNonCopyable       && ) = delete;
+};
+} // namespace issue_59
+
+CASE( "issue-58" )
+{
+    static_assert( !std::is_copy_constructible<issue_59::NonMovableNonCopyable>::value, "is not copy constructible" );
+    static_assert( !std::is_move_constructible<issue_59::NonMovableNonCopyable>::value, "is not move constructible" );
+
+    nonstd::expected<issue_59::NonMovableNonCopyable, issue_59::NonMovableNonCopyable> expected;
+    nonstd::expected<issue_59::NonMovableNonCopyable, issue_59::NonMovableNonCopyable> unexpected( nonstd::unexpect_t{} );
+
+    EXPECT(  expected.has_value()   );
+    EXPECT( !unexpected.has_value() );
+}
+
+#if !nsel_USES_STD_EXPECTED && nsel_P2505R >= 3
+CASE( "invoke" )
+{
+    struct A {
+      int x;
+      constexpr int get() const { return x; }
+      constexpr int get2(char) const { return x; }
+    };
+    static_assert( nonstd::expected_lite::detail::invoke( &A::x, A{21} ) == 21, "" );
+    EXPECT( nonstd::expected_lite::detail::invoke( &MoveOnly::x, MoveOnly(42) ) == 42 );
+    constexpr A lval{ 7 };
+    static_assert( nonstd::expected_lite::detail::invoke( &A::x, lval ) == 7, "" );
+    A mut_lval{ 12 };
+    std::reference_wrapper<A> ref{ mut_lval };
+    const std::reference_wrapper<const A> cref{ lval };
+    EXPECT( nonstd::expected_lite::detail::invoke( &A::x, ref ) == 12 );
+    EXPECT( nonstd::expected_lite::detail::invoke( &A::x, cref ) == 7 );
+    static_assert( nonstd::expected_lite::detail::invoke(&A::x, &lval) == 7, "" );
+    static_assert( nonstd::expected_lite::detail::invoke(&A::get, &lval) == 7, "" );
+    static_assert( nonstd::expected_lite::detail::invoke(&A::get, A{77}) == 77, "" );
+    EXPECT( nonstd::expected_lite::detail::invoke(&A::get, ref) == 12 );
+    EXPECT( nonstd::expected_lite::detail::invoke(&A::get, cref) == 7 );
+    static_assert( nonstd::expected_lite::detail::invoke(&A::get2, &lval, 'a') == 7, "" );
+    static_assert( nonstd::expected_lite::detail::invoke(&A::get2, A{77}, 'a') == 77, "" );
+    EXPECT( nonstd::expected_lite::detail::invoke(&A::get2, ref, 'a') == 12 );
+    EXPECT( nonstd::expected_lite::detail::invoke(&A::get2, cref, 'a') == 7 );
+}
+#endif // nsel_USES_STD_EXPECTED && nsel_P2505R >= 3
+
 // -----------------------------------------------------------------------
 //  using as optional
 
